#!/usr/bin/env python3
# Wavefront to C++ source
# Thomas Perl <m@thp.io>; 2014-01-03
#
# Usage: python3 obj2cpp.py outfile.h file1.obj file2.obj ...
#

import sys
import os

sys.stdout = open(sys.argv[1], 'w')

def triangles(vertices):
    for i in range(len(vertices)-2):
        yield from vertices[i:i+3]

def parse_obj(filename):
    vertices, normals, faces = [], [], []
    obj = {
        'filename': filename,
        'vertices': vertices,
        'normals': normals,
        'faces': faces,
    }
    for line in open(filename):
        line = line.strip()
        cmd, *args = line.split(' ')
        if cmd == 'v':
            assert len(args) == 3, line
            vertices.append(tuple(float(x) for x in args))
        elif cmd == 'vn':
            assert len(args) == 3, line
            normals.append(tuple(float(x) for x in args))
        elif cmd == 'f':
            for vertex in triangles(args):
                vtx, _, nor = (int(x)-1 if x else 0 for x in vertex.split('/'))
                faces.append(vertices[vtx] + normals[nor])

    return obj

def generate_vertex_data(faces):
    for face in faces:
        for value in face:
            yield ' %+.6ff,' % value
        yield '\n'

def obj2cpp(obj, all_vertices):
    faces = len(obj['faces'])
    structname = os.path.splitext(os.path.basename(obj['filename']))[0]
    print('struct obj_mesh_t %(structname)s = { %(start)d, %(count)d };' % {
        'structname': structname,
        'start': len(all_vertices),
        'count': len(obj['faces']),
    })
    all_vertices.extend(obj['faces'])

print("""

// created using: "python3 %s"

struct obj_mesh_t {
    int start; // start of first vertex
    int count; // count of vertices
};
""" % (' '.join(sys.argv)))

all_vertices = []
for filename in sorted(sys.argv[2:]):
    obj2cpp(parse_obj(filename), all_vertices)

print("""
float artwork_vertexdata[] = {
%s
};
""" % ''.join(generate_vertex_data(all_vertices)))
