/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/

#ifndef TETREPETETE_GLDRAW_H
#define TETREPETETE_GLDRAW_H

#include "drawcontext.h"
#include "game.h"
#include "color.h"

class OpenGLDrawContext : public DrawContext
{
    public:
        OpenGLDrawContext(Game *game);
        ~OpenGLDrawContext();

        /* GL callbacks */
        void initializeGL();
        void resizeGL(int width, int height);
        void paintGL();

        /* Helper functions */
        void mapCoords(int &x, int &y);

    private:
        /* Private drawing functions */
        void setColor(enum Color color, float opacity=1.f);
        void drawTriangle(float r, float g, float b, float a,
                int x1, int y1, int x2, int y2, int x3, int y3);
        void drawBrick(enum Color color, int x, int y, float rotation, float opacity, bool empty, float darken);

    public:
        /* DrawContext */
        void draw_begin();
        void draw_background(enum Color c1, enum Color c2);
        void draw_brick(enum Color color, int x, int y, float rotation=0.f);
        void draw_empty(int x, int y, bool right, bool bottom);
        void draw_fading(float value, int x, int y);
        void draw_score(long ticks, long score, int x, int y, int w, int h);
        void draw_title(int x, int y, int w, int h);
        void draw_shade();
        void draw_rectangle(enum Color color, int x, int y, int w, int h, float opacity=1.f);
        void draw_rectangle(float r, float g, float b, float a, int x, int y, int w, int h);
        void draw_triangle(enum Color color, int x1, int y1, int x2, int y2, int x3, int y3, float opacity=1.f);
        void draw_number(enum Color color, long number, int x, int y, int w, int h);
        void draw_text(enum Color color, const char *text, int x, int y, int w, int h, int flags);
        void translate(int x, int y);
        void draw_end();

    protected:
        Game *m_game;
        bool m_rotated;
        int m_width;
        int m_height;
        float m_scale;
};

#endif /* TETREPETETE_GLDRAW_H */
