/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/

#ifndef TETREPETETE_GLOCKE_H
#define TETREPETETE_GLOCKE_H

#include "qwtype.h"

struct GLockeAlphaTexturePriv;

class GLockeAlphaTexture {
public:
    GLockeAlphaTexture(char *data, int w, int h);
    ~GLockeAlphaTexture();

private:
    GLockeAlphaTexturePriv *priv;

    friend class GLocke;
};

class GLocke {
public:
    static void init();
    static bool resize(int &w, int &h);

    static void begin();
    static void clearDepth();
    static void end();

    static void draw(GLockeAlphaTexture *texture);
    static void draw3d(int start, int count);
    static void draw25d(int vertices, float *vertexptr);
    static void draw2d(int vertices, float *vertexptr);
    static void color(float r, float g, float b, float a);

    static void pushMatrix();
    static void popMatrix();

    static void translate(float x, float y, float z);
    static void rotate(float angle, float x, float y, float z);
    static void scale(float x, float y, float z);

    static void uploadMeshData(void *data, size_t len);
    static void lightPosition(float x, float y, float z);

    static const qwMatrix &modelviewMatrix();

    // size = number of position components in vertex data (usually 3)
    // stride = bytes between two consecutive vertices (or 0 for tightly packed)
    // vertexptr = pointer to first vertex data entry
    // count = number of vertices for which the modelview matrix should be applied
    static void applyModelView(int size, int stride, float *vertexptr, int count);
};

#endif /* TETREPETETE_GLOCKE_H */
