/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/


#include "glocke.h"

#include <math.h>

#include "artwork.h"

#define DRAW_OBJECT(x) GLocke::draw3d(x.start, x.count)

#include "rabbit.h"

#include "color.h"
#include "constants.h"

void
xxColor4f(float r, float g, float b, float a)
{
    if (!Constants::FULL_VERSION) {
        r = g = b = (r + g + b) / 3.;
    }
    GLocke::color(r, g, b, a);
}

void
setSymbolicColor(enum Color color)
{
    float r, g, b;
    RGBColor::map(color, r, g, b);
    r *= .1;
    g *= .1;
    b *= .1;
    xxColor4f(r, g, b, 1.);
}

void
drawRabbit(long time, int score)
{

    GLocke::pushMatrix();
    GLocke::translate(-1., 0., 0.);

    GLocke::pushMatrix();
    drawWing(20.*cosf(time*.004));
    GLocke::popMatrix();

    GLocke::pushMatrix();
    GLocke::scale(-1., 1., 1.);
    GLocke::translate(-2., 0., 0.);
    drawWing(20.*cosf(time*.004));
    GLocke::popMatrix();

    GLocke::pushMatrix();
    GLocke::translate(1., 0., 0.);
    GLocke::rotate(15.*sinf(time*.003), 0., 0., 1.);
    GLocke::translate(-1., 0., 0.);
    drawHead();
    GLocke::rotate(-5.*sinf(time*.003), 0., 0., 1.);
    drawBoard(score);
    GLocke::popMatrix();

    GLocke::popMatrix();
}

void
drawHead()
{
    GLocke::translate(1., 0., 0.);

    xxColor4f(0., 0., 0., 1.);
    DRAW_OBJECT(head_black);

    xxColor4f(0.465, 0.629, 0.800, 1.);
    DRAW_OBJECT(head_blue);

    xxColor4f(0.8, 0.8, 0.8, 1.);
    DRAW_OBJECT(head_gray);

    xxColor4f(1., 1., 1., 1.);
    DRAW_OBJECT(head_white);
}

void
drawWing(float rotation)
{
    GLocke::rotate(90., 1., 0., 0.);

    GLocke::translate(1., 0., -.2);
    GLocke::rotate(rotation, 0, 1, 0);
    GLocke::translate(-1., 0., .2);

    GLocke::translate(.1, 0., 0.);
    GLocke::translate(0., -.05, 0.);
    GLocke::rotate(-5., 1., 0., 0.);

    xxColor4f(1., 1., 1., 1.);
    DRAW_OBJECT(wing_body);

    xxColor4f(0.182, 0.467, 0.800, 1.);
    DRAW_OBJECT(wing_line1);

    xxColor4f(0.361, 0.800, 0.665, 1.);
    DRAW_OBJECT(wing_line2);

    xxColor4f(0.125, 0.229, 0.800, 1.);
    DRAW_OBJECT(wing_line3);
}

void
drawFrameCount(int cur)
{
    GLocke::rotate(90., 1., 0., 0.);

    bool shown = false;
    while (cur || !shown) {
        shown = true;
        switch (cur % 10) {
            case 0: DRAW_OBJECT(number0); break;
            case 1: DRAW_OBJECT(number1); break;
            case 2: DRAW_OBJECT(number2); break;
            case 3: DRAW_OBJECT(number3); break;
            case 4: DRAW_OBJECT(number4); break;
            case 5: DRAW_OBJECT(number5); break;
            case 6: DRAW_OBJECT(number6); break;
            case 7: DRAW_OBJECT(number7); break;
            case 8: DRAW_OBJECT(number8); break;
            case 9: DRAW_OBJECT(number9); break;
        }

        GLocke::translate(-.4, 0., 0.);

        cur /= 10;
    }
}


void
drawBoard(int score)
{
    GLocke::translate(0., -2.8, 0.);

    GLocke::pushMatrix();

    GLocke::scale(.9, .9, .3);

    GLocke::pushMatrix();
    GLocke::scale(.9, .9, 1.);
    setSymbolicColor(COLOR_PRIMARY);
    DRAW_OBJECT(board_sheet);
    GLocke::popMatrix();

    setSymbolicColor(GREY);
    DRAW_OBJECT(board_border);

    xxColor4f(1., 1., 1., 1.);
    GLocke::scale(.6, .6, 1.);
    GLocke::translate(1.05, -.28, .05);
    drawFrameCount(score);
    GLocke::popMatrix();

}

void
rabbitSetupLighting()
{
    GLocke::lightPosition(-3.0, 1.0, 3.0);
    GLocke::uploadMeshData(artwork_vertexdata, sizeof(artwork_vertexdata));
}

float
calcRotation(float time, int i)
{
    float rotation = fmod(time*.2-i*6, 360.);

    if (rotation > 180.) {
        rotation -= 360;
    }

    if (fabs(rotation) < 180.) {
        float f = powf(fabs(rotation) / 180., 5.);
        return rotation * f + (1.-f) * 30.;
    } else {
        return rotation;
    }
}

#define DRAW_TITLE_CHAR(c, i, time) \
    i++; \
    GLocke::pushMatrix(); \
    GLocke::translate(i*.45, 0., 0.); \
    if (i>=4) GLocke::translate(-.1, 0., 0.); \
    if (i==6) GLocke::translate(-.05, 0., 0.); \
    GLocke::rotate(calcRotation(time, i), 1., .6, 0.2); \
    DRAW_OBJECT(c); \
    GLocke::popMatrix();

void
drawTitle(float time)
{
    GLocke::pushMatrix();

    GLocke::rotate(-90., 1., 0., 0.);

    int i = 0;

    DRAW_TITLE_CHAR(title_big_t, i, time);
    DRAW_TITLE_CHAR(title_e, i, time);
    DRAW_TITLE_CHAR(title_small_t, i, time);
    DRAW_TITLE_CHAR(title_r, i, time);
    DRAW_TITLE_CHAR(title_e, i, time);

    GLocke::translate(-1.8, 2., .5);
    DRAW_TITLE_CHAR(title_p, i, time);
    DRAW_TITLE_CHAR(title_e, i, time);
    DRAW_TITLE_CHAR(title_small_t, i, time);
    DRAW_TITLE_CHAR(title_e, i, time);
    DRAW_TITLE_CHAR(title_small_t, i, time);
    DRAW_TITLE_CHAR(title_e, i, time);

    GLocke::popMatrix();
}

