/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/


#include "renderedtext.h"

#include "vincent.h"

#include <string.h>

RenderedText::RenderedText(const char *text)
    : texture(0)
{
    int scale = 1;
    int count = strlen(text);
    int spacing = 0;

    m_width = ((8+spacing)*count - spacing) * scale;
    m_height = 8*scale;

    int w = 1;
    while (w < m_width) w *= 2;
    int h = 1;
    while (h < m_height) h *= 2;

    char *buffer = new char[w * h];
    memset(buffer, 0x00, w * h);

    int x = 0;
    for (int i=0; i<count; i++) {
        for (int row=0; row<8; row++) {
            int data = vincent_data[text[i]][row];
            int column;
            for (column=0; column<8; column++) {
                if (data & (1 << (7-column))) {
                    int offset = (x + column + row * w);
                    buffer[offset] = 0xFF;
                }
            }
        }

        x += (8 + spacing) * scale;
    }

    texture = new GLockeAlphaTexture(buffer, w, h);

    delete [] buffer;
}

RenderedText::~RenderedText()
{
    delete texture;
}

void
RenderedText::render()
{
    GLocke::draw(texture);
}
