/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/


#include "tetrepetete_jni.h"

#include "gldraw.h"
#include "game.h"

class AndroidDrawContext : public OpenGLDrawContext
{
    public:
        AndroidDrawContext(const char *data_dir)
            : OpenGLDrawContext(new Game(data_dir))
        {
        }

        void
        nativeTouch(int event, int x, int y)
        {
            mapCoords(x, y);

            switch (event) {
                case ACTION_DOWN:
                    m_game->touch(true, x, y);
                    break;
                case ACTION_MOVE:
                    m_game->motion(x, y);
                    break;
                case ACTION_UP:
                    m_game->touch(false, x, y);
                    break;
                default:
                    break;
            }
        }

        ~AndroidDrawContext()
        {
            delete m_game;
        }
};

static AndroidDrawContext *draw = NULL;

void nativeInit(JNIEnv *env, jobject o, jstring path);
void nativeResize(JNIEnv *env, jobject o, jint width, jint height);
void nativeRender(JNIEnv *env, jobject o);
void nativeTouch(JNIEnv *env, jobject o, jint event, jfloat x, jfloat y, jint finger);

// javap -private -s io.thp.tetrepetete.android.GameRenderer
static JNINativeMethod game_renderer_methods[] = {
    "nativeInit", "(Ljava/lang/String;)V", (void*)nativeInit,
    "nativeResize", "(II)V", (void*)nativeResize,
    "nativeRender", "()V", (void*)nativeRender,
};

// javap -private -s io.thp.tetrepetete.android.GameView
static JNINativeMethod game_view_methods[] = {
    "nativeTouch", "(IFFI)V", (void*)nativeTouch,
};

jint
JNI_OnLoad(JavaVM *vm, void *reserved)
{
    JNIEnv *env;

    vm->GetEnv((void**)&env, JNI_VERSION_1_2);

    jclass game_renderer_class = env->FindClass("io/thp/tetrepetete/android/GameRenderer");
    env->RegisterNatives(game_renderer_class, game_renderer_methods, 3);

    jclass game_view_class = env->FindClass("io/thp/tetrepetete/android/GameView");
    env->RegisterNatives(game_view_class, game_view_methods, 1);

    return JNI_VERSION_1_2;
}

void
nativeInit(JNIEnv *env, jobject o, jstring path)
{
    const char *data_dir = env->GetStringUTFChars(path, 0);

    draw = new AndroidDrawContext(data_dir);
    draw->initializeGL();

    env->ReleaseStringUTFChars(path, data_dir);
}

void
nativeResize(JNIEnv *env, jobject o, jint width, jint height)
{
    if (draw) {
        draw->resizeGL(width, height);
    }
}

void
nativeRender(JNIEnv *env, jobject o)
{
    if (draw) {
        draw->paintGL();
    }
}

void
nativeTouch(JNIEnv *env, jobject o, jint event, jfloat x, jfloat y, jint finger)
{
    if (draw) {
        draw->nativeTouch(event, x, y);
    }
}

