/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/

#ifndef BOARD_H
#define BOARD_H

#include "pattern.h"
#include "brick.h"
#include "drawcontext.h"
#include "scene.h"
#include "recording.h"

#include "randomizer.h"
#include "colorscheme.h"
#include "score.h"

#include <vector>

struct Field {
    bool m_occupied;
    int m_pattern;
    bool m_fading;
    float m_fading_value;
    int m_y_offset;

    Field()
        : m_occupied(false),
          m_pattern(-1),
          m_fading(false),
          m_fading_value(0.),
          m_y_offset(0)
    {
    }
};

class Game;

class Board : public Scene {
    public:
        Board(Game *game);
        ~Board();

        void playback(const char *filename);
        void playback_event(RecordingEvent event);

        static void get_score(const char *filename, long *score, long *lines);

        /* Scene: */
        virtual enum SceneType type() { return GAMEPLAY; }
        void touch(enum EventType event, int x, int y);
        void keyboard(bool pressed, bool repeat, enum Game::Key key);
        void render(DrawContext *context);
        void resize(int width, int height);
        void event(enum Game::Event event);
        void tick();
        void transition(float value);

    private:
        void rotate(int direction);
        void move(int direction);

        bool fall(bool is_drop=false);
        void drop();
        void new_pattern(int id=-1);

        void fade();

        void start_fast();
        void stop_fast();

        void freeze();

        struct Field *at(int x, int y);

        bool check_collision(Pattern *pattern);
        void remove_rows();
        void brick_pos(int column, int row, int &x, int &y);

        /* XXX: Remove and replace with level */
        float m_speedup;

        /* Window size and board position */
        int m_window_width;
        int m_window_height;
        int m_x_offset;
        int m_y_offset;

        /* Ticks for score animation */
        long m_ticks;

        /* Brick falling handling */
        int m_fall_ticks;
        float m_fall_accumulator;

        /* Helper classes */
        Randomizer m_randomizer;
        ColorScheme m_colorscheme;
        Score m_score;

        /* Fields on the board */
        struct Field *m_fields;
        Pattern *m_current_pattern;
        std::vector<Brick*> m_falling_bricks;

        /* Touch input handling */
        Point m_start_touch;
        long m_start_touch_time;
        Point m_last_touch;
        bool m_moved;
        long m_last_drop;
	bool m_can_drop;

        /* Demo recording */
        bool m_playback;
        Recording m_recording;

        /* GUI Animations */
        int m_pause_button_y;
};

#endif
