/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/


#include "color.h"

#include "constants.h"

void
RGBColor::map(enum Color color, int &r, int &g, int &b)
{
    r = g = b = 0;

    switch (color) {
        case WHITE:
            r = g = b = 240;
            break;
        case RED:
            r = 0xef;
            g = 0x29;
            b = 0x29;
            break;
        case GREEN:
            r = 0x8a;
            g = 0xe2;
            b = 0x34;
            break;
        case BLUE:
            r = 0x72;
            g = 0x9f;
            b = 0xcf;
            break;
        case CYAN:
            r = 0x13;
            g = 0xb2;
            b = 0xb8;
            break;
        case PURPLE:
            r = 0xa4;
            g = 0x1d;
            b = 0xa9;
            break;
        case LIGHT_GREEN:
            r = 0x6f;
            g = 0xde;
            b = 0x8b;
            break;
        case ORANGE:
            r = 0xff;
            g = 0x4d;
            b = 0x12;
            break;
        case LIGHT_BLUE:
            r = 0x84;
            g = 0x8d;
            b = 0xff;
            break;
        case PINK:
            r = 0xea;
            g = 0x14;
            b = 0x79;
            break;
        case MAROON:
            r = 0x77;
            g = 0x11;
            b = 0x11;
            break;
        case LIGHT_ORANGE:
            r = 0xff;
            g = 0x6f;
            b = 0x47;
            break;
        case GREY:
        case SHADOW:
            r = g = b = 0x40;
            break;
        default:
            break;
    }

    if (!Constants::FULL_VERSION) {
        /* Yes. This is the line. The free-to-grey line. Go figure. */
        r = g = b = (r + g + b) / 3;
    }
}

void
RGBColor::map(enum Color color, float &r, float &g, float &b)
{
    int ri, gi, bi;
    map(color, ri, gi, bi);
    r = ri / 255.;
    g = gi / 255.;
    b = bi / 255.;
}

