/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/


#include "confirmscene.h"

#include <stdio.h>

ConfirmScene::ConfirmScene(Game *game, Scene *parent, const char *message,
        const char *affirmative, enum Game::Event affirmative_action,
        const char *negative, enum Game::Event negative_action)
    : Scene(game),
      m_parent(parent),
      m_message(message),
      m_affirmative(affirmative),
      m_affirmative_action(affirmative_action),
      m_negative(negative),
      m_negative_action(negative_action),
      m_button_width(0),
      m_button_left(0),
      m_button_height(0),
      m_button_top(0),
      m_affirmative_y(0),
      m_negative_y(0)
{
}

void
ConfirmScene::touch(enum EventType event, int x, int y)
{
    if (event == Scene::UP) {
        if (x >= m_button_left && x <= m_button_left + m_button_width) {
            if (y >= m_affirmative_y && y <= m_affirmative_y + m_button_height) {
                m_game->event(m_affirmative_action);
            } else if (y >= m_negative_y && y <= m_negative_y + m_button_height) {
                m_game->event(m_negative_action);
            }
        }
    }
}

void
ConfirmScene::keyboard(bool pressed, bool repeat, enum Game::Key key)
{
}

void
ConfirmScene::render(DrawContext *context)
{
    m_parent->render(context);
    context->draw_shade();

    int x = m_button_left;
    int y = m_button_top;

    context->draw_text(WHITE, m_message, x, y, m_button_width, m_button_height, DrawContext::TEXT_CENTER);

    y = m_affirmative_y;
    context->draw_button(COLOR_PRIMARY, m_affirmative, x, y, m_button_width, m_button_height);

    y = m_negative_y;
    context->draw_button(COLOR_PRIMARY, m_negative, x, y, m_button_width, m_button_height);
}

void
ConfirmScene::resize(int width, int height)
{
    m_button_width = m_game->m_width - Constants::CONFIRM_SCENE_BORDER * 2.;
    m_button_left = Constants::CONFIRM_SCENE_BORDER;

    m_button_height = m_game->m_height / 5.;
    m_button_top = m_game->m_height / 2. - Constants::CONFIRM_SCENE_BORDER - m_button_height;

    m_affirmative_y = m_button_top + m_button_height + Constants::CONFIRM_SCENE_BORDER;
    m_negative_y = m_affirmative_y + m_button_height + Constants::CONFIRM_SCENE_BORDER;
}

void
ConfirmScene::tick()
{
}

void
ConfirmScene::event(enum Game::Event event)
{
}

