/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/

#ifndef TETREPETETE_DRAWCONTEXT_H
#define TETREPETETE_DRAWCONTEXT_H

#include "constants.h"
#include "color.h"

struct DrawContext
{
    virtual void draw_begin() = 0;

    virtual void draw_background(enum Color c1, enum Color c2) = 0;

    /* Board */
    virtual void draw_brick(enum Color color, int x, int y, float rotation=0.f) = 0;
    virtual void draw_empty(int x, int y, bool right, bool bottom) = 0;
    virtual void draw_fading(float value, int x, int y) = 0;
    virtual void draw_score(long ticks, long score, int x, int y, int w, int h) = 0;

    /* GUI */
    virtual void draw_shade() = 0;
    virtual void draw_rectangle(enum Color color, int x, int y, int w, int h, float opacity=1.f) = 0;
    virtual void draw_triangle(enum Color color, int x1, int y1, int x2, int y2, int x3, int y3, float opacity=1.f) = 0;
    virtual void draw_number(enum Color color, long number, int x, int y, int w, int h) = 0;
    virtual void draw_title(int x, int y, int w, int h) = 0;

    enum TextFlags {
        TEXT_NONE = 0,
        TEXT_CENTER,
    };
    virtual void draw_text(enum Color color, const char *text, int x, int y, int w, int h, int flags) = 0;

    virtual void translate(int x, int y) = 0;

    virtual void draw_end() = 0;

    /* Higher-level draw functions using functions from above */
    void draw_pause_button(int x, int y, int w, int h) {
        float pause_line_width = w / 5.;
        float pause_line_spacing = pause_line_width / 2.;
        float pause_line_height = pause_line_width * 2. + pause_line_spacing;

        float pause_line_top = y + (h - pause_line_height) / 2.;
        float pause_line_left = x + (w - pause_line_height) / 2.;

        draw_rectangle(BLACK, x, y, w, h, .5);
        draw_rectangle(WHITE, pause_line_left, pause_line_top,
                pause_line_width, pause_line_height);
        draw_rectangle(WHITE, pause_line_left + pause_line_width + pause_line_spacing, pause_line_top,
                pause_line_width, pause_line_height);
    }

    void draw_back_button(int x, int y, int w, int h) {
        float back_rect_height = w / 5;
        float back_size = back_rect_height * 2.5;

        float back_rect_left = x + (w / 2.);
        float back_rect_top = y + (h - back_rect_height) / 2.;
        float back_rect_width = back_size / 2.;

        float back_tri_left = x + (w - back_size) / 2.;
        float back_tri_right = back_rect_left;
        float back_tri_top = y + (w - back_size) / 2.;
        float back_tri_bottom = back_tri_top + back_size;
        float back_tri_center = y + (w / 2.);

        draw_rectangle(BLACK, x, y, w, h, .5);
        draw_triangle(WHITE, back_tri_left, back_tri_center, back_tri_right, back_tri_top,
                back_tri_right, back_tri_bottom);
        draw_rectangle(WHITE, back_rect_left, back_rect_top, back_rect_width, back_rect_height);
    }

    virtual void draw_button(enum Color color, const char *text, int x, int y, int w, int h)
    {
        draw_rectangle(color, x, y, w, h, .9);
        draw_text(WHITE, text, x, y, w, h, DrawContext::TEXT_CENTER);
    }
};

#endif /* TETREPETETE_DRAWCONTEXT_H */
