/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/

#ifndef TETREPETETE_GUISCENE_H
#define TETREPETETE_GUISCENE_H

#include "scene.h"

#include <vector>

struct GuiElement;

class GuiScene : public Scene {
    public:
        GuiScene(Game *game);
        ~GuiScene();

        void touch(enum EventType event, int x, int y);
        void keyboard(bool pressed, bool repeat, enum Game::Key key);
        void render(DrawContext *context);
        void resize(int width, int height);
        void tick();
        void event(enum Game::Event event);

        void send_event(enum Game::Event event);

        void transition(float value);

        void set_visible(GuiElement *element, bool visible);

    protected:
        GuiElement *add_button(const char *text, int x, int y, int width, int height, enum Game::Event action);
        GuiElement *add_number(long number, int x, int y, int width, int height);
        GuiElement *add_text(const char *text, int x, int y, int width, int height, int flags, enum Color color);

    private:
        std::vector<GuiElement*> m_elements;
        GuiElement *m_back_button;
};

#endif /* TETREPETETE_GUISCENE_H */
