/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/


#include "menuscene.h"

#include "util.h"

#include <stdlib.h>

MenuScene::MenuScene(Game *game)
    : GuiScene(game),
      m_replay_button(NULL)
{
    int x = 20;
    int w = m_game->m_width - 2*x;
    int h = 100;

    int buttons = 3;
    int spacing = 20;

    int content_height = spacing * (buttons-1) + h * buttons;
    int y = (m_game->m_height - content_height) / 2.;

    add_button("Start Game", x, y, w, h, Game::START_GAME);
    y += h + spacing;
    add_button("Highscores", x, y, w, h, Game::SHOW_HIGHSCORES);
    y += h + spacing;
    m_replay_button = add_button("Replay", x, y, w, h, Game::START_PLAYBACK);
}

void
MenuScene::event(enum Game::Event event)
{
    if (event == Game::SCENE_EVENT_PUSH || Game::SCENE_EVENT_EXPOSE) {
        set_visible(m_replay_button, Util::file_exists(m_game->m_last_recording));
    }

    GuiScene::event(event);
}

void
MenuScene::render(DrawContext *context)
{
    GuiScene::render(context);

    context->draw_text(WHITE, "v" VERSION,
            m_game->m_width / 2,
            m_game->m_height - 50,
            m_game->m_width / 2, 50,
            DrawContext::TEXT_CENTER);
}
