/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/


#include "pattern.h"
#include "constants.h"

#include <stdlib.h>

const char *PATTERNS[] = {
    "#c#",
    " # ",
    NULL,

    "#c##",
    NULL,

    "#x",
    "##",
    NULL,

    "#c#",
    "#",
    NULL,

    "#c#",
    "  #",
    NULL,

    " c#",
    "##",
    NULL,

    "#c",
    " ##",
    NULL,

    NULL,
};

XY pattern_encoded[PATTERNS_COUNT][ROTATIONS_COUNT][PIECES_COUNT];

bool Pattern::encoded = false;

void
Pattern::encode()
{
    if (encoded) return;
    encoded = true;

    int pattern = 0;
    int offset = 0;

    while (true) {
        XY center;
        XY pieces[PIECES_COUNT];
        bool can_rotate = true;
        int piece = 0;
        int row = 0;

        while (PATTERNS[offset]) {
            const char *line = PATTERNS[offset];
            int column = 0;

            while (line[column]) {
                if (line[column] == 'c' || line[column] == 'x') {
                    center.x = column;
                    center.y = row;
                    can_rotate = (line[column] == 'c');
                } else if (line[column] == '#') {
                    pieces[piece].x = column;
                    pieces[piece].y = row;
                    piece++;
                }

                column++;
            }

            offset++;
            row++;
        }

        for (piece=0; piece<PIECES_COUNT; piece++) {
            XY xy = pieces[piece];
            xy.x -= center.x;
            xy.y -= center.y;

            int rotation;
            for (rotation=0; rotation<ROTATIONS_COUNT; rotation++) {
                pattern_encoded[pattern][rotation][piece] = xy;

                if (can_rotate) {
                    int tmp;
                    tmp = xy.x;
                    xy.x = -xy.y;
                    xy.y = tmp;
                }
            }
        }

        offset++;
        if (!PATTERNS[offset]) break;

        pattern++;
    }

}

Pattern::Pattern(int id)
    : m_id(id),
      m_x(Constants::BOARD_WIDTH/2),
      m_y(-2),
      m_rotation(0),
      m_kick_offset(0)
{
    encode();
}

int
Pattern::pattern()
{
    return m_id;
}

void
Pattern::rotate(int direction)
{
    m_kick_offset = 0;

    m_rotation = (m_rotation + direction) % 4;
    while (m_rotation < 0) m_rotation += 4;

    int min_x = m_x;
    int max_x = m_x;

    for (int i=0; i<4; i++) {
        int column, row;
        xy(i, column, row);
        if (column < min_x) {
            min_x = column;
        }
        if (column > max_x) {
            max_x = column;
        }
    }

    if (min_x < 0) {
        m_kick_offset = -(min_x);
    } else if (max_x >= Constants::BOARD_WIDTH) {
        m_kick_offset = -(max_x - Constants::BOARD_WIDTH + 1);
    }
}

void
Pattern::fall(int direction)
{
    m_y += direction;
}

void
Pattern::move(int direction)
{
    if ((m_kick_offset < 0 && direction < 0) ||
            (m_kick_offset > 0 && direction > 0)) {
        m_x += m_kick_offset;
        m_kick_offset = 0;
    }

    m_x += direction;
}

