/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/

#ifndef PATTERN_H
#define PATTERN_H

#include "constants.h"


#define PATTERNS_COUNT 7
#define ROTATIONS_COUNT 4
#define PIECES_COUNT 3

struct _XY {
    int x;
    int y;
};

typedef struct _XY XY;

extern XY pattern_encoded[PATTERNS_COUNT][ROTATIONS_COUNT][PIECES_COUNT];


class Pattern {
    public:
        Pattern(int id);

        void rotate(int direction);
        void fall(int direction);
        void move(int direction);

        int pattern();

        inline void xy(int idx, int &x, int &y)
        {
            x = m_x + m_kick_offset;
            y = m_y;

            if (idx > 0) {
                x += pattern_encoded[m_id][m_rotation][idx-1].x;
                y += pattern_encoded[m_id][m_rotation][idx-1].y;
            }
        }

        inline bool contains(int x, int y)
        {
            int i;
            int dx, dy;

            for (i=0; i<4; i++) {
                xy(i, dx, dy);
                if (dx == x && dy == y) return true;
            }

            return false;
        }

    private:
        int m_id;
        int m_x;
        int m_y;
        int m_rotation;
        int m_kick_offset;

        static void encode();
        static bool encoded;
};

#endif
