/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/


#include "recording.h"

#include "board.h"

#include <stdio.h>

Recording::Recording()
    : m_events(),
      m_last_ticks(0),
      m_playback_ticks(0),
      m_playback_index(0)
{
}

bool
Recording::playback_step(Board *board, long ticks)
{
    // If ticks is negative, play everything

    m_playback_ticks += ticks;

    while (m_playback_index < m_events.size()) {
        RecordingEvent event = m_events[m_playback_index];
        if (m_playback_ticks > event.m_ticks || ticks < 0) {
            board->playback_event(event);
            m_playback_index++;
            m_playback_ticks -= event.m_ticks;
        } else {
            break;
        }
    }

    if (m_playback_index >= m_events.size()) {
        board->playback_event(RecordingEvent(RecordingEvent::QUIT, 0, 0));
        return false;
    }

    return true;
}

void
Recording::load(const char *filename)
{
    FILE *fp = fopen(filename, "rb");

    m_events.clear();
    m_playback_ticks = 0;
    m_playback_index = 0;

    RecordingEvent event;
    if (fp != NULL) {
        while (fread(&event, sizeof(RecordingEvent), 1, fp) == 1) {
            m_events.push_back(event);
        }
        fclose(fp);
    }
}

void
Recording::save(const char *filename)
{
    FILE *fp = fopen(filename, "wb");
    if (fp != NULL) {
        for (int i=0; i<m_events.size(); i++) {
            fwrite(&(m_events[i]), sizeof(RecordingEvent), 1, fp);
        }
        fclose(fp);
    }
}

