/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/

#ifndef TETREPETETE_RECORDING_H
#define TETREPETETE_RECORDING_H

#include <vector>

class Board;

struct RecordingEvent {
    enum EventType {
        INVALID,
        ROTATE,
        MOVE,
        DROP,
        FALL,
        NEW_PATTERN,
        QUIT,
    };

    RecordingEvent()
        : m_type(INVALID),
          m_data(0),
          m_ticks(0)
    {}

    RecordingEvent(enum EventType type, signed char data, unsigned char ticks)
        : m_type(type),
          m_data(data),
          m_ticks(ticks)
    {}

    enum EventType m_type;
    signed char m_data;
    unsigned char m_ticks;
};

class Recording {
    public:
        Recording();

        void
        add(enum RecordingEvent::EventType type, long ticks)
        {
            add(type, 0, ticks);
        }

        void
        add(enum RecordingEvent::EventType type, signed char data, long ticks)
        {
            unsigned char diff = ticks - m_last_ticks;
            m_events.push_back(RecordingEvent(type, data, diff));
            m_last_ticks = ticks;
        }

        bool
        playback_step(Board *board, long ticks);

        void
        load(const char *filename);

        void
        save(const char *filename);

    private:
        std::vector<RecordingEvent> m_events;

        /* Recording */
        long m_last_ticks;

        /* Playback */
        long m_playback_ticks;
        int m_playback_index;
};

#endif /* TETREPETETE_RECORDING_H */
