/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/

#ifndef TETREPETETE_SCENE_H
#define TETREPETETE_SCENE_H

#include "game.h"
#include "drawcontext.h"
#include "color.h"
#include "constants.h"

class Scene {
    public:
        Scene(Game *game);
        virtual ~Scene();

        enum SceneType {
            GENERIC, /* Generic - default for all scenes */
            GAMEPLAY, /* Gameplay - lock swipe gestures */
            OVERLAY, /* Overlay - disable animations */
        };

        enum EventType {
            DOWN,
            MOTION,
            UP,
        };

        virtual enum SceneType type() { return GENERIC; }
        virtual void touch(enum EventType event, int x, int y) = 0;
        virtual void keyboard(bool pressed, bool repeat, enum Game::Key key) = 0;
        virtual void render(DrawContext *context) = 0;
        virtual void resize(int width, int height) = 0;
        virtual void tick() = 0;
        virtual void event(enum Game::Event event) = 0;
        virtual void transition(float value) {}

    protected:
        Game *m_game;
};

#endif /* TETREPETETE_SCENE_H */
