/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/


#include "tetrepetete.h"

#include "score.h"
#include "constants.h"

Score::Score()
    : m_score(0),
      m_lines(0),
      m_lines_speedup_last(0),
      m_last_event(Game::NONE),
      m_combo_multiplier(0)
{
}

void
Score::event(enum Game::Event event)
{
    switch (event) {
        case Game::PATTERN_FROZEN:
            m_score += Constants::FREEZE_POINTS;
            m_combo_multiplier = 0;
            break;

        case Game::SINGLE_LINE_CLEAR:
            m_lines += 1;
            m_score += Constants::SINGLE_LINE_POINTS * m_combo_multiplier;
            break;

        case Game::DOUBLE_LINE_CLEAR:
            m_lines += 2;
            m_score += Constants::DOUBLE_LINE_POINTS * m_combo_multiplier;
            break;

        case Game::TRIPLE_LINE_CLEAR:
            m_lines += 3;
            m_score += Constants::TRIPLE_LINE_POINTS * m_combo_multiplier;
            break;

        case Game::PETETE_LINE_CLEAR:
            m_lines += 4;
            m_score += Constants::PETETE_LINE_POINTS * m_combo_multiplier;
            break;

        default:
            tetrepetete_assert(0);
    }

    m_combo_multiplier++;
    m_last_event = event;
}

