/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/


#include "transition.h"

#include "game.h"
#include "constants.h"

Transition::Transition(Game *game, Scene *from, Scene *to, bool is_push)
    : m_game(game),
      m_ticks(0),
      m_from(from),
      m_to(to),
      m_is_push(is_push)
{
}

void
Transition::tick()
{
    m_ticks++;
}

bool
Transition::done()
{
    return m_ticks >= Constants::SCENE_TRANSITION_TICKS;
}

void
Transition::render(DrawContext *context, float partial_tick)
{
    float value = (float)(m_ticks + partial_tick) / (float)Constants::SCENE_TRANSITION_TICKS;

    /* Overshooting */
    float s = 1.;
    value = 1. - ((s+1) * powf(1. - value, 3.) - s * powf(1. - value, 2.));

    if (m_is_push) {
        context->translate(-m_game->m_width * value * 2, 0);
    } else {
        context->translate(m_game->m_width * value, 0);
    }

    m_from->render(context);
    m_from->transition(1.-value);

    context->translate(m_game->m_width * value, 0);

    if (m_is_push) {
        context->translate(m_game->m_width, 0);
    } else {
        context->translate(-m_game->m_width * 2, 0);
    }

    m_to->render(context);
    m_to->transition(value);
}

