/**
 *  Tetrepetete - A lightweight and portable game with blocks.
 *  http://thp.io/2013/tetrepetete/
 *  Copyright (C) 2012, 2013, 2014, 2015 Thomas Perl <m@thp.io>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 **/


#include "tetrepetete.h"

#include "util.h"

#include <stdlib.h>
#include <sys/stat.h>
#include <string.h>
#include <stdio.h>
#include <limits.h>

#include <algorithm>

long
Util::now()
{
    static struct timeval tv0;
    static bool inited = false;
    if (!inited) {
        gettimeofday(&tv0, NULL);
        inited = true;
    }

    struct timeval tv;
    if (gettimeofday(&tv, NULL) == 0) {
        return (tv.tv_sec - tv0.tv_sec) * 1000 + (tv.tv_usec - tv0.tv_usec) / 1000;
    }

    return 0;
}

const char *
Util::get_data_dir()
{
    static char data_dir[PATH_MAX+1];
    static bool inited = false;

    if (!inited) {
        strcpy(data_dir, getenv("HOME"));
        strcat(data_dir, "/.config/tetrepetete");
    }

    return data_dir;
}

void
Util::recursive_mkdir(const char *path)
{
    struct stat st;

    if (stat(path, &st) == 0) {
        if (S_ISDIR(st.st_mode)) {
            return;
        } else {
            tetrepetete_assert(0);
        }
    }

    char *tmp = strdup(path);
    for (int i=0; i<strlen(path); i++) {
        if (i > 0 && tmp[i] == '/') {
            tmp[i] = '\0';

            if (stat(tmp, &st) == 0) {
                tetrepetete_assert(S_ISDIR(st.st_mode));
            } else {
                tetrepetete_assert(mkdir(tmp, 0700) == 0);
            }

            tmp[i] = '/';
        }
    }
    free(tmp);

    tetrepetete_assert(mkdir(path, 0700) == 0);
}

char *
Util::join_path(const char *path, const char *filename)
{
    char *result = (char*)malloc(strlen(path) + 1 + strlen(filename) + 1);
    strcpy(result, path);
    strcat(result, "/");
    strcat(result, filename);
    return result;
}

bool
Util::file_exists(const char *filename)
{
    struct stat st;
    if (stat(filename, &st) == 0) {
        return !S_ISDIR(st.st_mode);
    }

    return false;
}

